#include <windows.h>

#include <dinput.h>
#include "DXUtil.h"

#pragma comment(lib, "dinput8.lib")
#pragma comment(lib, "dxguid.lib")

//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------
struct EFFECTS_NODE
{
    LPDIRECTINPUTEFFECT pDIEffect;
    DWORD               dwPlayRepeatCount;
};

LPDIRECTINPUT8			g_pDI       = NULL;         
LPDIRECTINPUTDEVICE8	g_pFFDevice = NULL;

#define _FFE_NUMMAX        16

EFFECTS_NODE			g_EffectsList[_FFE_NUMMAX];

static INT				g_FFE_Num = 0;

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        103
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1003
#define _APS_NEXT_SYMED_VALUE           102
#endif
#endif
//-----------------------------------------------------------------------------
// Function-prototypes
//-----------------------------------------------------------------------------

HRESULT FFEInit( HWND hDlg );
HRESULT FFEBye();
HRESULT FFEReadFile(TCHAR strFileName[MAX_PATH], int &maxeffnum);
HRESULT FFEPlayEffects( INT FFE_Id );
BOOL CALLBACK EnumFFDevicesCallback( LPCDIDEVICEINSTANCE pDDI, LPVOID pvRef );
BOOL CALLBACK EnumAndCreateEffectsCallback( LPCDIFILEEFFECT pDIFileEffect, LPVOID pvRef );
VOID    EmptyEffectList();

//-------------------------------------------------------------------------------
//	DLLGg[
//-------------------------------------------------------------------------------
BOOL APIENTRY DllMain( HINSTANCE hInstance , DWORD fdwReason , PVOID pvReserved )
{
	switch( fdwReason )
	{
	case DLL_PROCESS_ATTACH:
		break;
	case DLL_THREAD_ATTACH:
		break;
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}

//-----------------------------------------------------------------------------
// Name: EnumFFDevicesCallback()
// Desc: Get the first enumerated force feedback device
//-----------------------------------------------------------------------------
BOOL CALLBACK EnumFFDevicesCallback( LPCDIDEVICEINSTANCE pDDI, VOID* pvRef )
{
    if( FAILED( g_pDI->CreateDevice( pDDI->guidInstance, &g_pFFDevice, NULL ) ) )
        return DIENUM_CONTINUE; // If failed, try again

    // Stop when a device was successfully found
    return DIENUM_STOP;
}

//-----------------------------------------------------------------------------
// Name: EnumAndCreateEffectsCallback()
// Desc: Create the effects as they are enumerated and add them to the 
//       linked list, g_EffectsList
//-----------------------------------------------------------------------------
BOOL CALLBACK EnumAndCreateEffectsCallback( LPCDIFILEEFFECT pDIFileEffect, LPVOID pvRef )
{   
    HRESULT hr;
    LPDIRECTINPUTEFFECT pDIEffect = NULL;

    // Create the file effect
    if( FAILED( hr = g_pFFDevice->CreateEffect( pDIFileEffect->GuidEffect, 
                                                pDIFileEffect->lpDiEffect, 
                                                &pDIEffect, NULL ) ) )
    {
	//   OutputDebugString( "Could not create force feedback effect on this device.\n" );
        return DIENUM_CONTINUE;
    }

	EFFECTS_NODE* pEffectNode = &g_EffectsList[g_FFE_Num];

	// Fill the pEffectNode up
    ZeroMemory( pEffectNode + g_FFE_Num, sizeof( EFFECTS_NODE ) );
    pEffectNode->pDIEffect         = pDIEffect;
    pEffectNode->dwPlayRepeatCount = 1;

	g_FFE_Num ++;
	if (g_FFE_Num < _FFE_NUMMAX){
	  return DIENUM_CONTINUE;
	}

    return DIENUM_STOP;
}

//-----------------------------------------------------------------------------
// Name: FFEInit()
// Desc: Initialize the DirectInput variables.
//-----------------------------------------------------------------------------
HRESULT FFEInit( HWND hDlg )
{
    HRESULT hr;

	// Setup the g_EffectsList circular linked list
   // EmptyEffectList();

    if( FAILED( hr = DirectInput8Create( GetModuleHandle(NULL), DIRECTINPUT_VERSION, 
                                         IID_IDirectInput8, (VOID**)&g_pDI, NULL ) ) )
        return hr;

    // Get the first enumerated force feedback device
    if( FAILED( hr = g_pDI->EnumDevices( 0, EnumFFDevicesCallback, 0, 
                                         DIEDFL_ATTACHEDONLY | 
                                         DIEDFL_FORCEFEEDBACK ) ) )
        return hr;

	//FFI
    if( g_pFFDevice == NULL ){
        return S_FALSE;
    }

    // Set the data format
    if( FAILED( hr = g_pFFDevice->SetDataFormat( &c_dfDIJoystick ) ) )
        return hr;

    // Set the coop level
    if( FAILED( hr = g_pFFDevice->SetCooperativeLevel( hDlg, DISCL_EXCLUSIVE | 
                                                             DISCL_BACKGROUND ) ) )
        return hr;

	// Disable auto-centering spring
    DIPROPDWORD dipdw;
    dipdw.diph.dwSize       = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj        = 0;
    dipdw.diph.dwHow        = DIPH_DEVICE;
    dipdw.dwData            = FALSE;

    if( FAILED( hr = g_pFFDevice->SetProperty( DIPROP_AUTOCENTER, &dipdw.diph ) ) )
        return hr;

    // Acquire the device
    if( FAILED( hr = g_pFFDevice->Acquire() ) )
        return hr;

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: FFEBye()
// Desc: Initialize the DirectInput variables.
//-----------------------------------------------------------------------------
HRESULT FFEBye()
{
    // Release any DirectInputEffect objects.
    if( g_pFFDevice ) 
    {
        EmptyEffectList();
        g_pFFDevice->Unacquire();
        SAFE_RELEASE( g_pFFDevice );
    }

    // Release any DirectInput objects.
    SAFE_RELEASE( g_pDI );

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: EmptyEffectList()
// Desc: Goes through the circular linked list and releases the effects, 
//       and deletes the nodes
//-----------------------------------------------------------------------------
VOID EmptyEffectList()
{
	int cnt;
    EFFECTS_NODE* pEffectDelete;

	for(cnt = 0;cnt < _FFE_NUMMAX; cnt++){
        pEffectDelete = &g_EffectsList[cnt];
        SAFE_RELEASE( pEffectDelete->pDIEffect );
		ZeroMemory( pEffectDelete, sizeof( EFFECTS_NODE ) );
	}

	g_FFE_Num = 0;
}


//-----------------------------------------------------------------------------
// Name: OnReadFile()
// Desc: Reads a file contain a collection of DirectInput force feedback 
//       effects.  It creates each of effect read in and stores it 
//       in the linked list, g_EffectsList.
//-----------------------------------------------------------------------------
HRESULT FFEReadFile(TCHAR strFileName[MAX_PATH], int &maxeffnum)
{
    HRESULT hr;
	maxeffnum = 0;

    EmptyEffectList();

    // Enumerate the effects in the file selected, and create them in the callback
    if( FAILED( hr = g_pFFDevice->EnumEffectsInFile( strFileName, 
                                                     EnumAndCreateEffectsCallback, 
                                                     NULL, DIFEF_MODIFYIFNEEDED ) ) )
        return hr;

    // If list of effects is empty, then we haven't been able to create any effects
    if( g_FFE_Num == 0 ){ //Unable to create any effects.
        return S_FALSE;
    }

	maxeffnum = g_FFE_Num;
    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: OnPlayEffects()
// Desc: Plays all of the effects enumerated in the file 
//-----------------------------------------------------------------------------
HRESULT FFEPlayEffects( INT FFE_Id )
{
	if ((0 <= FFE_Id ) && (FFE_Id < g_FFE_Num)){
		EFFECTS_NODE* pEffectNode = &g_EffectsList[FFE_Id];
		LPDIRECTINPUTEFFECT pDIEffect   = NULL;
		HRESULT             hr;

		// Play all of the effects enumerated in the file 
		pDIEffect = pEffectNode->pDIEffect;
		if ( NULL != pDIEffect ){
			if( FAILED( hr = pDIEffect->Start( 1, 0 ) ) )
				return hr;
		}

		return S_OK;
    }else{
		return S_FALSE;
	}
}